package at.jku.risc.stout.tgau;

/*
 import com.google.gwt.event.dom.client.*;
 import com.google.gwt.core.client.EntryPoint;
 import com.google.gwt.user.client.ui.*;
 import com.google.gwt.dom.client.*;
 */

import java.io.PrintStream;
import java.io.StringReader;

import at.jku.risc.stout.tgau.algo.AntiUnify;
import at.jku.risc.stout.tgau.algo.AntiUnifySystem;
import at.jku.risc.stout.tgau.algo.DebugLevel;
import at.jku.risc.stout.tgau.algo.RigidityFnc;
import at.jku.risc.stout.tgau.algo.RigidityFncSubsequence;
import at.jku.risc.stout.tgau.data.InputParser;
import at.jku.risc.stout.tgau.data.NodeFactory;
import at.jku.risc.stout.tgau.data.TermGraph;
import at.jku.risc.stout.tgau.util.ControlledException;

/**
 * HelloWorld application.
 */
public class AntiUnifyGWT implements EntryPoint {

	private static final String PRINT_HEAD = "";
	private static final String PRINT_TAIL = "DONE";

	private final PrintStream out = new PrintStream(System.out) {
		StringBuilder buffer = new StringBuilder();
		long count = 0L;

		public void print(int b) {
			buffer.appendCodePoint(b);
			if (b == '\n') {
				count++;
				flush();
			}
		}

		public void print(char c) {
			buffer.append(c);
			if (c == '\n') {
				count++;
				flush();
			}
		}

		public void print(Object obj) {
			print(String.valueOf(obj));
		}

		public void println(Object obj) {
			println(String.valueOf(obj));
		}

		public void println(String str) {
			print(str);
			println();
		}

		public void print(String str) {
			buffer.append(str);
		}

		public void println() {
			print('\n');
		}

		public void flush() {
			String lineNum = "000000" + count + ": ";
			if (buffer.length() > 0)
				display(lineNum.substring(lineNum.length() - 8, lineNum.length()) + buffer.toString());
			buffer.setLength(0);
		}

		public void close() {
			flush();
			count = 0L;
		}
	};
	private final PrintStream info = new PrintStream(System.out) {
		StringBuilder buffer = new StringBuilder();

		public void print(int b) {
			buffer.appendCodePoint(b);
			if (b == '\n')
				flush();
		}

		public void print(char c) {
			buffer.append(c);
			if (c == '\n')
				flush();
		}

		public void print(Object obj) {
			print(String.valueOf(obj));
		}

		public void println(Object obj) {
			println(String.valueOf(obj));
		}

		public void println(String str) {
			print(str);
			println();
		}

		public void print(String str) {
			buffer.append(str);
		}

		public void println() {
			print('\n');
		}

		public void flush() {
			if (buffer.length() > 0)
				display("INFO:  " + buffer.toString(), "gray");
			buffer.setLength(0);
		}

		public void close() {
			flush();
		}
	};
	private final PrintStream err = new PrintStream(System.out) {
		StringBuilder buffer = new StringBuilder();

		public void print(int b) {
			buffer.appendCodePoint(b);
			if (b == '\n')
				flush();
		}

		public void print(char c) {
			buffer.append(c);
			if (c == '\n')
				flush();
		}

		public void print(Object obj) {
			print(String.valueOf(obj));
		}

		public void println(Object obj) {
			println(String.valueOf(obj));
		}

		public void println(String str) {
			print(str);
			println();
		}

		public void print(String str) {
			buffer.append(str);
		}

		public void println() {
			print('\n');
		}

		public void flush() {
			if (buffer.length() > 0)
				display("ERROR: " + buffer.toString(), "red");
			buffer.setLength(0);
		}

		public void close() {
			flush();
		}
	};

	public void btnClicked() {
		reset();
		if (PRINT_HEAD.length() > 0)
			display(PRINT_HEAD, "gray");
		try {
			String tg1 = getParameter("tg1");
			String tg2 = getParameter("tg2");
			DebugLevel debugLevel = DebugLevel.SIMPLE;
			try {
				debugLevel = DebugLevel.valueOf(getParameter("debugmode"));
			} catch (Exception ignored) {
			}
			boolean iterateAll = "true".equals(getParameter("iterateall"));
			boolean normalize = "true".equals(getParameter("normalize"));
			boolean splitRule = "true".equals(getParameter("splitrule"));
			boolean commutativeRules = "true".equals(getParameter("commutativerules"));
			// boolean justify = "true".equals(getParameter("justify"));
			int minLen = 1;
			try {
				minLen = Integer.parseInt(getParameter("minlen"));
			} catch (Exception e) {
			}
			RigidityFnc r;
			RigidityFncSubsequence rC = new RigidityFncSubsequence();
			rC.setMinLen(minLen);
			String rFncString = getParameter("rfnc");
			if (rFncString == null || rFncString.length() == 0)
				rFncString = "at.jku.risc.stout.tgau.algo.RigidityFncSubsequence";
			// Reflection doesn't work in GWT compiler
			if ("at.jku.risc.stout.tgau.algo.RigidityFncSubsequence".equals(rFncString)) {
				r = rC;
			} else if ("at.jku.risc.stout.tgau.algo.RigidityFncSubstring".equals(rFncString)) {
				r = new at.jku.risc.stout.tgau.algo.RigidityFncSubstring();
				r.setMinLen(minLen);
			} else {
				throw new ControlledException("Rigidity function invalid: " + rFncString);
			}

			if (splitRule) {
				if (commutativeRules)
					throw new ControlledException(
							"The split rule is not supported in combination with commutative functions.");
				info.println("Split rule is enabled");
			}
			if (commutativeRules) {
				info.println("Commutative rules are enabled");
				info.println("Commutative functions are applied to multisets of arguments");
				info.println("Double curly brackets are used to denote multisets");
			}
			info.println("Rigidity function = " + rFncString);
			info.println("Minimum alignment length = " + minLen);
			info.println("Iterate all possibilities = " + iterateAll);
			// info.println("Justify computed generalizations = " + justify);
			info.println("Output format = " + debugLevel);
			info.println();
			info.println("INPUT GRAPHS:");

			NodeFactory nf = new NodeFactory();
			if (commutativeRules) {
				if (getParameter("commutativesymbols") != null)
					nf.setCommutative(getParameter("commutativesymbols").trim().split("\\s+"));
				AntiUnifySystem.ENABLE_COMMUTATIVE_RULES = true;
			}
			AntiUnifySystem.ENABLE_SPLIT_RULE = splitRule;
			TermGraph g1 = new InputParser(nf).parseTermGraph(new StringReader(tg1), commutativeRules);
			TermGraph g2 = new InputParser(nf).parseTermGraph(new StringReader(tg2), commutativeRules);
			info.println("Term graph 1: " + g1);
			info.println("Term graph 2: " + g2);
			info.println();
			AntiUnify rau = new AntiUnify(r, rC, g1, g2, nf);
			rau.antiUnify(iterateAll, normalize, debugLevel, out);
		} catch (ControlledException ex) {
			err.println(ex.getMessage() + " (" + ex.getClass().getSimpleName() + ")");
		} catch (Throwable ex) {
			ex.printStackTrace(err);
		}

		info.flush();
		out.flush();
		err.flush();
		display(PRINT_TAIL, "gray");
	}

	public String getParameter(String name) {
		String ret = getTextVal(name);
		if (ret == null)
			ret = getSelectVal(name);
		if (ret == null)
			ret = getCheckedVal(name);
		return ret;
	}

	// GWT Comment Start
	private String getTextVal(String name) {
		if ("tg1".equals(name))
			return "x_0 = f(x_1,x_2,x_3,x_0,x_3,x_2, x_3); x_1 = g(x_1,x_2); x_2 = b; x_3 = a";
		if ("tg2".equals(name))
			return "y_0 = f(y_1, y_0, y_3),y_1 = g(y_1, y_2), y_2= b, y_3 = a";
		if ("debugmode".equals(name))
			return DebugLevel.VERBOSE.toString();
		return null;
	}

	private String getCheckedVal(String name) {
		return null;
	}

	private String getSelectVal(String name) {
		return null;
	}

	private static void display(Object obj, String... args) {
		System.out.print(obj);
	}

	private void reset() {
	}
	// GWT Comment END

/*
	private Document gwtDoc;
	private Element gwtContainer;
	private Button gwtButton;
	private NodeCollection<Element> gwtFormElems;

	public void onModuleLoad() {
		gwtContainer = RootPanel.get().getElement();
		gwtDoc = gwtContainer.getOwnerDocument();
		gwtContainer = gwtDoc.getElementById("gwtconsole");
		gwtButton = Button.wrap(gwtDoc.getElementById("gwtbutton"));
		gwtFormElems = gwtButton.getElement().<ButtonElement> cast().getForm()
				.getElements();
		gwtButton.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				btnClicked();
			}
		});
	}

	private void display(Object obj, String color) {
		Element gwtContainerParent = gwtContainer;
		gwtContainer = gwtDoc.createSpanElement();
		gwtContainer.getStyle().setColor(color);
		gwtContainerParent.appendChild(gwtContainer);
		display(obj);
		gwtContainer = gwtContainerParent;
	}

	private void display(Object obj) {
		gwtContainer.appendChild(gwtDoc.createTextNode(String.valueOf(obj)));
	}

	public String getTextVal(String name) {
		Element el = gwtFormElems.getNamedItem(name);
		if (InputElement.is(el)) {
			InputElement in = InputElement.as(el);
			if ("checkbox".equalsIgnoreCase(in.getType())
					|| "radio".equalsIgnoreCase(in.getType()))
				return null;
			return in.getValue();
		}
		if (TextAreaElement.is(el))
			return TextAreaElement.as(el).getValue();
		return null;
	}

	public String getCheckedVal(String name) {
		for (int i = 0, n = gwtFormElems.getLength(); i < n; i++) {
			Element el = gwtFormElems.getItem(i);
			if (InputElement.is(el)) {
				InputElement in = InputElement.as(el);
				if (in.isChecked() && name.equals(in.getName()))
					return in.getValue();
			}
		}
		return null;
	}

	public String getSelectVal(String name) {
		Element el = gwtFormElems.getNamedItem(name);
		if (SelectElement.is(el)) {
			SelectElement sel = SelectElement.as(el);
			if (sel.getLength() == 0)
				return null;
			int selIdx = sel.getSelectedIndex();
			if (selIdx == -1 && !sel.isMultiple())
				selIdx = 0;
			OptionElement selOpt = sel.getOptions().getItem(selIdx);
			return selOpt.getValue();
		}
		return null;
	}

	public void reset() {
		info.close();
		out.close();
		err.close();
		gwtContainer.removeAllChildren();
	}
*/
	public static void main(String[] args) {
		new AntiUnifyGWT().btnClicked();
	}
}
