/*
 * Copyright 2012-2018 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.tgau;

import java.io.IOException;
import java.io.StringReader;

import at.jku.risc.stout.tgau.algo.AntiUnify;
import at.jku.risc.stout.tgau.algo.AntiUnifySystem;
import at.jku.risc.stout.tgau.algo.DebugLevel;
import at.jku.risc.stout.tgau.algo.IllegalAlignmentException;
import at.jku.risc.stout.tgau.algo.RigidityFnc;
import at.jku.risc.stout.tgau.algo.RigidityFncSubsequence;
import at.jku.risc.stout.tgau.data.InputParser;
import at.jku.risc.stout.tgau.data.MalformedTermException;
import at.jku.risc.stout.tgau.data.NodeFactory;
import at.jku.risc.stout.tgau.data.TermGraph;

public class TestAntiunify {
	public static void main(String[] args) {
		try {
			// String g1 = "x0 = f(x1,x2,x3,x0,x3,x2, x3); x1 = g(x1,x2); x2 = b; x3 = a";
			// String g2 = "y0 = f(y1, y0, y3),y1 = g(y1, y2), y2= b, y3 = a";
			// String g1 = "x0 = f(x0,x0,x0); x1 = g(x1,x2); x2 = b; x3 = a";
			// String g2 = "y0 = f(y1, y0, y3),y1 = g(y1, y2), y2= b, y3 = a";
			// String g1 = "x0 = f(x0,x0,x0)";
			// String g2 = "y0 = f(y1, y0, y3),y1 = f(y2, y2), y2= b, y3 = a";
			// String g1 = "x=f(x,y), y=f(x,y)";
			// String g2 = "y=f(x,y), x=f(x,y)";
			// String g1 = "x=f(x,y,z), y=f(x,y,z), z=f(x,y)";
			// String g2 = "y=f(x,y,z), x=f(x,y,z), z=f(x,y)";
			// String g1 = "x=f(x,x,x,x,x,x,x,x,x,x,x,x,x,x,x,x)";
			// String g2 = "y=f(y,y,y,y,y,y,y,y,y,y,y,y,y)";
			// String g1 = "x=f(y,y,y), y=f(z,z,z), z=f(x,y,z)";
			// String g2 = "y=f(y,y,y)";
			String g1 = "x=f(x,y,z,u,v,w),y=f(x),z=f,u=f,v=f,w=f";
			String g2 = "x=f(x,x,x,x,x,y),y=f";
			RigidityFncSubsequence r = new RigidityFncSubsequence();
			test(g1, g2, r, r);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static void test(String in1, String in2, RigidityFnc r, RigidityFncSubsequence rC)
			throws IOException, IllegalAlignmentException, MalformedTermException {
		NodeFactory nf = new NodeFactory();
		nf.setCommutative("f");
		AntiUnifySystem.ENABLE_COMMUTATIVE_RULES = true;
		AntiUnifySystem.ENABLE_SPLIT_RULE = false;
		TermGraph g1 = new InputParser(nf).parseTermGraph(new StringReader(in1),
				AntiUnifySystem.ENABLE_COMMUTATIVE_RULES);
		TermGraph g2 = new InputParser(nf).parseTermGraph(new StringReader(in2),
				AntiUnifySystem.ENABLE_COMMUTATIVE_RULES);
		System.out.println(g1);
		System.out.println(g2);
		AntiUnify rau = new AntiUnify(r, rC, g1, g2, nf);
		rau.antiUnify(true, true, DebugLevel.VERBOSE, System.out);
		System.out.println(rau.getNormalizedSet().size() + " normalized results");
	}
}
