/*
 * Copyright 2012-2018 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.tgau.algo;

import java.io.IOException;
import java.io.Writer;
import java.util.List;

import at.jku.risc.stout.tgau.data.atom.HedgeVar;
import at.jku.risc.stout.tgau.data.atom.Term;
import at.jku.risc.stout.tgau.data.atom.Variable;
import at.jku.risc.stout.tgau.util.DataStructureFactory;

/**
 * This class represents an anti-unification problem (AUP) which consists of one
 * generalization variable (the most general generalization), and two
 * {@linkplain Term}s. The term nodes represent the left hand side and the right
 * hand side of the equation.
 * 
 * @author Alexander Baumgartner
 */
public class HedgeAUP extends BaseAUP {
	public HedgeVar generalizationVar;
	public List<Variable> left, right;
	public boolean commutative;

	/**
	 * Create an AUP with the given generalization variable and hedges.
	 * 
	 * @param commutative
	 */
	public HedgeAUP(HedgeVar generalizationVar, List<Variable> left, List<Variable> right, boolean commutative) {
		this.generalizationVar = generalizationVar;
		this.left = left;
		this.right = right;
		this.commutative = commutative;
	}

	@Override
	public void print(Writer out) throws IOException {
		generalizationVar.print(out);
		out.append(':').append(' ');
		DataStructureFactory.$.printCollection(left, commutative, out);
		out.append(PRINT_EQ_SEPARATOR);
		DataStructureFactory.$.printCollection(right, commutative, out);
	}

	@Override
	public Object getLeft() {
		return left;
	}

	@Override
	public Object getRight() {
		return right;
	}

	@Override
	public Variable getGeneralizationVar() {
		return generalizationVar;
	}

	@Override
	public boolean isCommutative() {
		return commutative;
	}
}
