/*
 * Copyright 2012-2018 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.tgau.algo;

import java.io.IOException;
import java.io.Writer;

import at.jku.risc.stout.tgau.data.atom.Term;
import at.jku.risc.stout.tgau.data.atom.TermVar;
import at.jku.risc.stout.tgau.data.atom.Variable;

/**
 * This class represents an anti-unification problem (AUP) which consists of one
 * generalization variable (the most general generalization), and two
 * {@linkplain Term}s. The term nodes represent the left hand side and the right
 * hand side of the equation.
 * 
 * @author Alexander Baumgartner
 */
public class TermAUP extends BaseAUP implements Comparable<TermAUP> {
	public TermVar generalizationVar;
	public TermVar left, right;

	/**
	 * Create an AUP with the given generalization variable and hedges.
	 */
	public TermAUP(TermVar generalizationVar, TermVar left, TermVar right) {
		this.generalizationVar = generalizationVar;
		this.left = left;
		this.right = right;
	}

	@Override
	public int hashCode() {
		return (997 * left.hashCode()) ^ (991 * right.hashCode());
	}

	@Override
	public boolean equals(Object o) {
		return o instanceof TermAUP && left == ((TermAUP) o).left && right == ((TermAUP) o).right;
	}

	@Override
	public int compareTo(TermAUP o) {
		return left == o.left ? right.compareTo(o.right) : left.compareTo(o.left);
	}

	@Override
	public void print(Writer out) throws IOException {
		generalizationVar.print(out);
		out.append(':').append(' ');
		left.print(out);
		out.append(PRINT_EQ_SEPARATOR);
		right.print(out);
	}

	@Override
	public Object getLeft() {
		return left;
	}

	@Override
	public Object getRight() {
		return right;
	}

	@Override
	public Variable getGeneralizationVar() {
		return generalizationVar;
	}
}
