/*
 * Copyright 2012-2018 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.tgau.data;

import java.io.IOException;
import java.io.Writer;

import at.jku.risc.stout.tgau.data.atom.TermAtom;
import at.jku.risc.stout.tgau.util.PrintableX;

/**
 * A pair of two term atoms.
 * 
 * @author Alexander Baumgartner
 */

public class TermAtomEquation extends PrintableX implements Comparable<TermAtomEquation> {
	public TermAtom left, right;
	public static String PRINT_EQ_SEPARATOR = " =^= ";

	public TermAtomEquation(TermAtom left, TermAtom right) {
		this.left = left;
		this.right = right;
	}

	@Override
	public int hashCode() {
		return (997 * left.hashCode()) ^ (991 * right.hashCode());
	}

	@Override
	public boolean equals(Object o) {
		return o instanceof TermAtomEquation && left == ((TermAtomEquation) o).left
				&& right == ((TermAtomEquation) o).right;
	}

	@Override
	public int compareTo(TermAtomEquation o) {
		return left == o.left ? right.compare(o.right) : left.compare(o.left);
	}

	@Override
	public void print(Writer out) throws IOException {
		left.print(out);
		out.append(PRINT_EQ_SEPARATOR);
		right.print(out);
	}

}
