/*
 * Copyright 2012-2018 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.tgau.util;

import java.util.Arrays;

/**
 * Tiny implementation of a dynamic sized generic array.
 * 
 * @author Alexander Baumgartner
 */
public class DynamicArray<T> {
	public T[] data;
	public int size = 0;

	/**
	 * Creates an empty dynamic sized array with the given initial capacity.
	 */
	public DynamicArray(T[] init) {
		this.data = init;
	}

	public void add(T elem) {
		if (size == data.length) {
			if (size < 8)
				data = Arrays.copyOf(data, 16);
			else
				data = Arrays.copyOf(data, size + (size >> 1));
		}
		data[size++] = elem;
	}

	public void remove(int idx) {
		int numMoved = --size - idx;
		if (numMoved > 0)
			System.arraycopy(data, idx + 1, data, idx, numMoved);
	}
}